/**
 * JASMINe Deploy ME [Managed Element]
 * Copyright (C) 2008 Bull S.A.S.
 * Copyright (C) 2008 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Configurator.java 5307 2009-09-15 18:39:10Z alitokmen $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.deployme.configApply;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.logging.Logger;

import org.ow2.jasmine.deployme.generated.Configuration;
import org.ow2.jasmine.deployme.generated.Topology.Domains.Domain;
import org.ow2.jasmine.deployme.utils.ServerDomain;

/**
 * Configure a jonas server
 * 
 * @author Remy Bresson
 */
public class Configurator {

    /**
     * The logger
     */
    protected static Logger logger = Logger.getLogger(Configurator.class.getName());

    /**
     * Recursive directory deleter
     */
    public static void rmdir(final File directory) {
        if (directory != null) {
            if (directory.isDirectory()) {
                for (File element : directory.listFiles()) {
                    if (element.isDirectory()) {
                        rmdir(element);
                    } else {
                        element.delete();
                    }
                }
            }
            directory.delete();
        }
    }

    /**
     * Set the configuration of this cluster daemon, only used for tests
     * 
     * @param domainName Cluster daemon domain name
     * @param config The config of the cluster daemon
     * @param jonasRoot JONAS_ROOT directory to use
     */
    public static void setClusterAConfiguration(final String domainName, final Domain.ClusterDaemons.ClusterDaemon cluster,
        final String jonasRoot) {
        logger.info("Starting configuration set up for cluster " + cluster.getName() + " in directory "
            + cluster.getJonasBasePrefix());
        if (cluster.getJonasBasePrefix() != null) {
            File prefix = new File(cluster.getJonasBasePrefix());
            rmdir(prefix);

            // Make sure the pathname is absolute
            cluster.setJonasBasePrefix(prefix.getAbsolutePath());
        }

        cluster.setJonasRoot(jonasRoot);
        ConfigClusterApply apply = new ConfigClusterApply(jonasRoot);
        apply.setClusterConf(domainName, cluster);

        apply.execute();
        createScripts(cluster.getJavaHome(), null, cluster.getJonasRoot(), cluster.getJonasBasePrefix(), domainName, cluster
            .getName());
        logger.info("Cluster " + cluster.getName() + " configured successfully");
    }

    /**
     * Set the configuration of this cluster daemon
     * 
     * @param domainName Cluster daemon domain name
     * @param config The config of the cluster daemon
     */
    public static void setClusterAConfiguration(final String domainName, final Domain.ClusterDaemons.ClusterDaemon cluster) {
        if (cluster.getJonasRoot() == null) {
            throw new IllegalArgumentException("The topology file doesn't define any JONAS_ROOT");
        }
        setClusterAConfiguration(domainName, cluster, cluster.getJonasRoot());
    }

    /**
     * Set the configuration of this server, only used for tests
     * 
     * @param config The config
     * @param sd The server and his domain
     * @param jonasRoot path to the jonas root to use
     */
    public static void setAConfiguration(final Configuration config, final ServerDomain sd, final String jonasRoot) {
        logger.info("Starting configuration set up for server " + sd + " in directory "
            + config.getGlobalOptions().getJonasBase());
        if (config.getGlobalOptions() != null) {
            config.getGlobalOptions().setJonasRoot(jonasRoot);
            if (config.getGlobalOptions().getJonasBase() != null) {
                if (config.getGlobalOptions().getJonasBase().equals(config.getGlobalOptions().getJonasRoot())) {
                    throw new IllegalArgumentException("JONAS_BASE must be different from JONAS_ROOT for server " + sd);
                }
                File jonasBase = new File(config.getGlobalOptions().getJonasBase());
                rmdir(jonasBase);

                // Make sure the pathname is absolute
                config.getGlobalOptions().setJonasBase(jonasBase.getAbsolutePath());
            }
        }

        ConfigJonasApply apply = new ConfigJonasApply(jonasRoot);
        apply.setGlobalOptions(sd.getServer().getName(), sd.getDomain().getName(), config.getGlobalOptions());
        apply.setServiceDb(config.getDb());
        apply.setServiceDiscovery(config.getDiscovery());
        apply.setServiceHa(config.getHa());
        apply.setServiceMail(config.getMail());
        apply.setServiceWeb(config.getWeb());
        apply.setServiceWsdlPublisher(config.getWsdlPublisher());
        apply.setSecurityManager(config.getSecurityManager());
        apply.setEjbClusteringConfiguration(config.getEjbClustering());
        apply.setJmsConfiguration(config.getJms());
        apply.setJdbcConfiguration(config.getJdbc());

        apply.execute();
        createScripts(config.getGlobalOptions().getJdk(), config.getGlobalOptions().getJavaOpts(), config.getGlobalOptions()
            .getJonasRoot(), config.getGlobalOptions().getJonasBase(), sd.getDomain().getName(), sd.getServer().getName());
        logger.info("Server " + sd + " configured successfully");
    }

    /**
     * Set the configuration of this server
     * 
     * @param config The config
     * @param sd The server and his domain
     */
    public static void setAConfiguration(final Configuration config, final ServerDomain sd) {
        if (config.getGlobalOptions() == null || config.getGlobalOptions().getJonasRoot() == null) {
            throw new IllegalArgumentException("The topology file doesn't define any JONAS_ROOT");
        }
        setAConfiguration(config, sd, config.getGlobalOptions().getJonasRoot());
    }

    /**
     * Create the setenv, start and stop scripts
     * 
     * @param javaHome JAVA_HOME
     * @param javaOpts JAVA_OPTS
     * @param jonasRoot JONAS_ROOT
     * @param jonasBase JONAS_BASE
     * @param jonasDomain JOnAS server domain
     * @param jonasName JOnAS server name
     */
    private static void createScripts(final String javaHome, String javaOpts, final String jonasRoot, final String jonasBase,
        String jonasDomain, String jonasName) {
        if (javaOpts == null) {
            javaOpts = "";
        }
        if (jonasName == null) {
            jonasName = "jonas";
        }
        if (jonasDomain == null) {
            jonasDomain = "jonas";
        }

        try {
            File f;
            FileWriter fw;

            f = new File(jonasBase, "setenv.bat");
            fw = new FileWriter(f, false);
            if (javaHome != null) {
                fw.write("SET JAVA_HOME=" + javaHome + "\r\n");
            }
            fw.write("SET JAVA_OPTS=%JAVA_OPTS% " + javaOpts + "\r\n\r\n");
            fw.write("SET JONAS_ROOT=" + jonasRoot + "\r\n");
            fw.write("SET JONAS_BASE=" + jonasBase + "\r\n");
            fw.write("SET JONAS_OPTS=%JONAS_OPTS% -Djonas.name=" + jonasName + " -Ddomain.name=" + jonasDomain + "\r\n");
            fw.write("SET PATH=" + jonasRoot + "\\bin;" + jonasRoot + "\\bin\\nt;%PATH%\r\n");
            fw.close();

            f = new File(jonasBase, "setenv");
            fw = new FileWriter(f, false);
            fw.write("#!/bin/sh\n");
            if (javaHome != null) {
                fw.write("JAVA_HOME=" + javaHome + "\n");
                fw.write("export JAVA_HOME\n");
            }
            fw.write("JAVA_OPTS=$JAVA_OPTS " + javaOpts + "\n");
            fw.write("export JAVA_OPTS\n\n");
            fw.write("JONAS_ROOT=" + jonasRoot + "\n");
            fw.write("export JONAS_ROOT\n");
            fw.write("JONAS_BASE=" + jonasBase + "\n");
            fw.write("export JONAS_BASE\n");
            fw.write("JONAS_OPTS=$JONAS_OPTS\\ -Djonas.name=" + jonasName + "\\ -Ddomain.name=" + jonasDomain + "\n");
            fw.write("export JONAS_OPTS\n");
            fw.write("PATH=" + jonasRoot + "/bin:" + jonasRoot + "/bin/unix:$PATH\n");
            fw.write("export PATH\n");
            fw.close();
            try {
                Process p = Runtime.getRuntime().exec("chmod a+x " + f.getAbsolutePath());
                p.waitFor();
            } catch (IOException ignored) {
                // This is Windows
            }
        } catch (Exception e) {
            throw new IllegalStateException("Cannot write the scripts", e);
        }
    }
}
